"""Miscellaneous utils."""

from typing import Any, Sized


def to_bool(value: Any) -> bool:
    """Convert an object to a bool value (True or False).

    :param value: any object that can be converted to a bool value.
    :return: True or False.
    """
    if isinstance(value, bool):
        return value
    if isinstance(value, str):
        if value.lower() in ("t", "true", "y", "yes"):
            return True
        if value.isdigit():
            return int(value) != 0
        return False
    if isinstance(value, int) and value != 0:
        return True
    if isinstance(value, Sized) and len(value) > 0:
        return True
    return False



"""Manipulating PDFs."""

import datetime
from pathlib import Path
import re
import time
from typing import Iterable
from pypdf import PdfWriter, PdfReader
import pdfplumber

FMT = "%Y%m%d"


def extract_pages(file: str, subfiles: dict[str, int | Iterable[int]]) -> None:
    """Extract pages from a PDF file and write into sub PDF file.

    :param file: The raw PDF file to extract pages from.
    :param subfiles: A dictionary specifying sub PDF files
        and their corresponding indexes (0-based) of pages from the raw PDF file.
        For example,
        the following code extract pages 0-4 as first.pdf,
        pages 5 and 7 as second.pdf,
        and page 6 as third.pdf from raw.pdf.
        .. highlight:: python
        .. code-block:: python

        from aiutil.pdf import extract_pages
        extract_pages("raw.pdf", {"first.pdf": range(5), "second.pdf": [5, 7], "third.pdf": 6})
    """
    with open(file, "rb") as fin:
        reader = PdfReader(fin)
        for subfile, indexes in subfiles.items():
            _extract_pages(reader, indexes, subfile)


def _extract_pages(
    reader: PdfReader, indexes: int | Iterable[int], output: str
) -> None:
    """A helper function for extract_pages.

    :param reader: A PdfFileReader object.
    :param indexes: Index (0-based) of pages to extract.
    :param output: The path of the sub PDF file to write the extracted pages to.
    """
    writer = PdfWriter()
    if isinstance(indexes, int):
        indexes = [indexes]
    for index in indexes:
        writer.add_page(reader.pages[index])
    with open(output, "wb") as fout:
        writer.write(fout)


def extract_text_first_page(path: str | Path) -> str:
    """Extract the text of the first page of a PDF file.

    :param path: The path of the PDF file.
    :return: The text of the first page.
    """
    with pdfplumber.open(path) as pdf:
        page = pdf.pages[0]
        return page.extract_text()


def _rename_puget_sound_energy(path: Path, text_first_page: str) -> Path:
    pattern = r"Issued: (\w+ \d{1,2}, \d{4})"
    m = re.search(pattern, text_first_page)
    if not m:
        raise RuntimeError(
            f"The pattern {pattern} is not found on the first page of the PDF bill."
        )
    date = datetime.datetime.strptime(m.group(1), "%B %d, %Y").strftime(FMT)
    path_new = path.with_name(f"pse_{date}.pdf")
    path.rename(path_new)
    return path_new


def _rename_bellevue_water(path: Path, text_first_page: str) -> Path:
    pattern = r"Bill Date: (\d{1,2}/\d{1,2}/\d{4})"
    m = re.search(pattern, text_first_page)
    if not m:
        raise RuntimeError(
            f"The pattern {pattern} is not found on the first page of the PDF bill."
        )
    date = datetime.datetime.strptime(m.group(1), "%m/%d/%Y").strftime(FMT)
    path_new = path.with_name(f"bellevue_water_{date}.pdf")
    path.rename(path_new)
    return path_new


def rename(pdf: str | Path) -> Path:
    """Rename a PDF file automatically based on its content.

    :param pdf: The path of the PDF file.
    :return: The path of the renamed PDF file.
    """
    if isinstance(pdf, str):
        pdf = Path(pdf)
    text = extract_text_first_page(pdf)
    pdf_new = pdf
    if "Puget Sound Energy" in text:
        pdf_new = _rename_puget_sound_energy(pdf, text)
    elif "MyUtilityBill.bellevuewa.gov" in text:
        pdf_new = _rename_bellevue_water(pdf, text)
    print(f"{pdf} ==> {pdf_new}")
    return pdf_new


def rename_dir(
    dir_: str | Path, seconds_wait: float = 0.1, seconds_total: float = 3600
):
    if isinstance(dir_, str):
        dir_ = Path(dir_)
    processed = set()
    time_begin = time.time()
    while True:
        if time.time() - time_begin > seconds_total:
            break
        time.sleep(seconds_wait)
        for path in dir_.iterdir():
            if path in processed:
                continue
            if path.suffix.lower() == ".pdf":
                processed.add(rename(path))



"""Set or overwrite key-value pairs in a sysctl configuration file (e.g., /etc/sysctl.conf)."""

import argparse
import re
from pathlib import Path
import subprocess as sp


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(
        description="Set/overwrite key to have value in /etc/sysctl.conf."
    )
    parser.add_argument("key", help="The key to set or overwrite.")
    parser.add_argument("value", help="The value to set.")
    parser.add_argument(
        "-p",
        "--path",
        dest="path",
        default="/etc/sysctl.conf",
        help="The path to the sysctl configuration file.",
    )
    parser.add_argument(
        "-a",
        "--apply",
        dest="apply",
        action="store_true",
        help="Whether to apply the changes by running 'sysctl -p'.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def _write_lines(lines: list[str], path: Path, apply: bool) -> None:
    with path.open("w", encoding="utf-8") as fout:
        fout.writelines(lines)
    if apply:
        sp.run(["sysctl", "-p", str(path)], check=True)


def etc_sysctl(
    key: str, value: str, path: str | Path = "/etc/sysctl.conf", apply: bool = False
) -> None:
    """Set/overwrite key to have value in /etc/sysctl.conf.

    :param key: The key to set or overwrite.
    :param value: The value (of key) to set.
    :param path: The path to the sysctl configuration file.
    :param apply: Whether to apply the changes by running 'sysctl -p'.
    """
    valid_values = {
        "kernel.perf_event_paranoid": ("-1", "0", "1", "2", "3"),
    }
    if key in valid_values:
        if value not in valid_values[key]:
            raise ValueError(
                f"Invalid value '{value}' for key '{key}'! "
                f"Valid values are {valid_values[key]}."
            )
    if isinstance(path, str):
        path = Path(path)
    if not path.exists():
        path.parent.mkdir(parents=True, exist_ok=True)
        return _write_lines([f"{key} = {value}\n"], path, apply)
    with path.open("r", encoding="utf-8") as fin:
        lines = fin.readlines()
    pattern = re.compile(rf"^\s*{re.escape(key)}\s*=")
    for i, line in enumerate(lines):
        if pattern.match(line):
            lines[i] = f"{key} = {value}\n"
            return _write_lines(lines, path, apply)
    lines.append(f"{key} = {value}\n")
    _write_lines(lines, path, apply)


def main():
    args = parse_args()
    etc_sysctl(args.key, args.value, path=args.path, apply=args.apply)


if __name__ == "__main__":
    main()



"""Add a GitHub repository and initialize a local Git repository with workflows."""

import argparse
import getpass
import os
import shutil
import sys
import tempfile
from pathlib import Path
from dulwich import porcelain
from github_rest_api import User, Organization


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(description="Add a GitHub repository.")
    parser.add_argument(
        "repo",
        help="The GitHub repo (in the format of owner/repo) to be created.",
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-u",
        "--user",
        dest="is_owner_user",
        action="store_true",
        help="The owner of the repo is a user.",
    )
    group.add_argument(
        "-o",
        "--org",
        "--organization",
        dest="is_owner_user",
        action="store_false",
        help="The owner of the repo is an organization.",
    )
    parser.add_argument(
        "-t",
        "--token",
        dest="token",
        default="",
        help="The GitHub token to use.",
    )
    parser.add_argument(
        "-p",
        "--public",
        dest="private",
        action="store_false",
        help="Whether to create the repository as public.",
    )
    parser.add_argument(
        "-l",
        "--lang",
        "--language",
        dest="language",
        default="",
        help="The language of the GitHub repository.",
    )
    parser.add_argument(
        "-d",
        "--dir",
        dest="dir",
        default="",
        help="The directory of the local Git repository.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def _create_remote_repo(
    repo: str, private: bool, token: str, is_owner_user: bool
) -> None:
    owner, r = repo.split("/")
    entity = (
        User(token=token, user=owner)
        if is_owner_user
        else Organization(token=token, org=owner)
    )
    entity.create_repository(name=r, private=private)
    print(f"\nCreated the GitHub repo https://github.com/{repo}.\n")


def _init_local_repo(repo: str, language: str, dir_: str, token: str) -> None:
    """Initialize a local Git repository.

    :param repo: The GitHub repo (in the format of owner/repo).
    :param language: The language of the GitHub repository.
    :param dir_: The directory of the local Git repository.
    :param token: The GitHub token to use.
    """
    repo_name = repo.split("/")[-1]
    path = Path(dir_) if dir_ else Path(repo_name)
    path.mkdir(parents=True, exist_ok=True)
    readme = path / "README.md"
    if not readme.exists():
        readme.write_text(f"# {repo_name}\n")
    porcelain.init(path=path)
    porcelain.add(repo=path)
    porcelain.commit(repo=path, message="first commit")
    # add GitHub Actions workflows
    _add_workflow(path, language)

    def _create_push_branch(branch: str):
        porcelain.branch_create(repo=path, name=branch)
        porcelain.checkout(repo=path, target=branch)
        porcelain.push(
            repo=path,
            remote_location=f"https://{token}@github.com/{repo}.git",
        )

    _create_push_branch("dev")
    _create_push_branch("main")
    porcelain.checkout(repo=path, target="dev")
    porcelain.branch_delete(repo=path, name="main")
    porcelain.branch_delete(repo=path, name="master")


def add_github_repo(
    repo: str, private: bool, language: str, is_owner_user: bool, dir_: str, token: str
) -> None:
    token = token or os.getenv("GITHUB_TOKEN", "")
    if not token:
        token = getpass.getpass("Please enter your GitHub token: ")
        if not token:
            sys.exit(
                "No GitHub token is provided (via $GITHUB_TOKEN, --token or at prompt)."
            )
    repo = repo.strip()
    _create_remote_repo(
        repo=repo, private=private, token=token, is_owner_user=is_owner_user
    )
    _init_local_repo(repo=repo, language=language, dir_=dir_, token=token)


def _add_workflow(path: Path, language: str) -> None:
    with tempfile.TemporaryDirectory() as dir_temp:
        porcelain.clone(
            "https://github.com/legendu-net/github_actions_scripts",
            dir_temp,
            depth=1,
        )
        dir_src: Path = Path(dir_temp)
        dir_dest = path / ".github" / "workflows"
        for yaml in dir_src.glob("*.yml"):
            shutil.copy2(yaml, dir_dest)
        # language specific workflows
        if not language:
            return
        dir_src = dir_src / language
        if not dir_src.exists():
            return
        for yaml in (dir_src / language).glob("*.yml"):
            shutil.copy2(yaml, dir_dest)


def main():
    args = parse_args()
    add_github_repo(
        repo=args.repo,
        private=args.private,
        language=args.language,
        is_owner_user=args.is_owner_user,
        dir_=args.dir,
        token=args.token,
    )


if __name__ == "__main__":
    main()



import argparse
from pathlib import Path
import re
import subprocess as sp
from typing import Iterable


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(
        description="Lint Python project or scripts using uv."
    )
    parser.add_argument(
        "pyscripts",
        nargs="*",
        default=(),
        help="An (optional) list of Python scripts to lint.",
    )
    parser.add_argument(
        "--ty",
        action="store_const",
        const="ty",
        default="pyright",
        dest="type_checker",
        help="Use ty as the type checker.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def uv_lint(pyscripts: Iterable[str] | Iterable[Path], type_checker: str) -> None:
    plus35 = "+" * 35
    line = "\n\n" + plus35 + "{}" + plus35 + "\n"
    if Path("pyproject.toml").exists():
        print(line.format(" pyproject-fmt "))
        sp.run("uv run pyproject-fmt pyproject.toml", shell=True, check=True)
        print(line.format(" ruff check "))
        sp.run("uv run ruff check", shell=True, check=True)
        print(line.format(" ruff format "))
        sp.run("uv run ruff format", shell=True, check=True)
        print(line.format(" ty "))
        sp.run("uv run ty check", shell=True, check=True)
        print(line.format(" deptry "))
        sp.run("uv run deptry .", shell=True, check=True)
        print(line.format(" pytest "))
        sp.run("uv run pytest", shell=True, check=True)
        return
    if not pyscripts:
        pyscripts = Path().glob("**/*.py")
    for pyscript in pyscripts:
        print(line.format(f" {pyscript} "))
        print(line.format(" ruff check "))
        sp.run(
            f"uv run --with ruff --with-requirements '{pyscript}' ruff check '{pyscript}'",
            shell=True,
            check=True,
        )
        print(line.format(" ruff format "))
        sp.run(
            f"uv run --with ruff --with-requirements '{pyscript}' ruff format '{pyscript}'",
            shell=True,
            check=True,
        )
        if type_checker == "ty":
            print(line.format(" ty "))
            sp.run(
                f"uv run --with ty --with-requirements '{pyscript}' ty check '{pyscript}'",
                shell=True,
                check=True,
            )
        else:
            print(line.format(" pyright "))
            sp.run(
                f"uv run --with pyright --with-requirements '{pyscript}' pyright '{pyscript}'",
                shell=True,
                check=True,
            )
        if not re.search(r"^\s*def test_\w+\(", Path(pyscript).read_text()):
            continue
        print(line.format(" pytest "))
        sp.run(
            f"uv run --with pytest --with-requirements '{pyscript}' pytest check '{pyscript}'",
            shell=True,
            check=True,
        )


def main():
    args = parse_args()
    uv_lint(args.pyscripts, args.type_checker)


if __name__ == "__main__":
    main()



"""Add a Python Script using the template."""

import argparse
import shutil
import subprocess as sp
from pathlib import Path


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(description="Add a Python script.")
    parser.add_argument("name", help="The name of the Python script.")
    parser.add_argument(
        "-d",
        "--dir",
        dest="dir",
        default=".",
        help="The directory ('.' by default) within which to create the Python script.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def add_pyscript(name: str, dir_: str) -> Path:
    name = name.replace(" ", "_").replace("-", "_")
    dir_pyscript = Path(dir_)
    dir_pyscript.mkdir(exist_ok=True)
    path = dir_pyscript / f"{name}.py"
    if path.exists():
        print(f"The Python script {path} already exists.\n")
    else:
        text = (
            (Path(__file__).parent / "pyscript.txt")
            .read_text()
            .replace("my_function", name)
        )
        path.write_text(text)
        print(f"Created the Python script {path}.\n")
    return path


def main():
    args = parse_args()
    path = add_pyscript(args.name, args.dir)
    vim = "nvim" if shutil.which("nvim") else "vim"
    sp.run([vim, path], check=True)


if __name__ == "__main__":
    main()



import os
import sys
import argparse
import tomllib
import getpass
from pathlib import Path
from dulwich.repo import Repo
from github_rest_api import Repository


def find_root(path: Path | None = None) -> Path | None:
    if path is None:
        path = Path.cwd()
    while path != path.parent:
        if (path / ".git").exists():
            return path
        path = path.parent
    return None


def get_version(root: Path) -> str:
    pyproject = root / "pyproject.toml"
    if pyproject.exists():
        with pyproject.open("rb") as f:
            data = tomllib.load(f)
            return data.get("project", {}).get("version")
    cargo = root / "Cargo.toml"
    if cargo.exists():
        for line in cargo.read_text().splitlines():
            if line.strip().startswith("version"):
                return line.split("=")[1].strip().strip('"')
    return ""


def get_repo(root: Path) -> str | None:
    pyproject = root / "pyproject.toml"
    if pyproject.exists():
        with pyproject.open("rb") as f:
            data = tomllib.load(f)
            repo_url = data.get("project", {}).get("urls", {}).get("Repository")
            if repo_url:
                return repo_url.split("github.com/")[-1].rstrip("/")
    # Fallback to git remote
    try:
        repo = Repo(root)
        config = repo.get_config()
        output = config.get((b"remote", b"origin"), b"url").decode().strip()
        if "github.com" in output:
            if output.startswith("git@"):
                return output.split("github.com:")[-1].replace(".git", "").rstrip("/")
            return output.split("github.com/")[-1].replace(".git", "").rstrip("/")
    except Exception:
        pass
    return None


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(
        description="Make a release of the project on GitHub."
    )
    parser.add_argument(
        "-b",
        "--branch",
        default="main",
        help="The branch (default to main) from which to make the release.",
    )
    parser.add_argument(
        "-t",
        "--tag",
        default="",
        help="The tag for the release. If not specified, the version from project configuration is used.",
    )
    parser.add_argument(
        "-n",
        "--notes",
        default="",
        help="Notes for the release. If not specified, it's auto generated.",
    )
    parser.add_argument(
        "--token",
        default="",
        help="GitHub token. If not specified, the GITHUB_TOKEN environment variable is used.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def release_on_github(token: str, branch: str, tag: str = "", notes: str = "") -> None:
    """Make a release of the project on GitHub.

    The command should be run under the root or a sub-dir of the project.

    :param branch: The branch from which to make the release.
    :param tag: The tag for the release.
        If not specified, the version from project configuration (pyproject.toml, Cargo.toml, etc) is used.
    :param notes: Notes for the release.
        If not specified, it's auto generated.
    :param token: GitHub token.
        If not specified, the GITHUB_TOKEN environment variable is used.
    """
    token = token or os.getenv("GITHUB_TOKEN", "")
    if not token:
        token = getpass.getpass("Please enter your GitHub token: ")
        if not token:
            sys.exit(
                "No GitHub token is provided (via $GITHUB_TOKEN, --token or at prompt)."
            )
    root = find_root()
    if not root:
        sys.exit("Could not find project root (no .git found).")

    if not tag:
        tag = get_version(root)
        if not tag:
            sys.exit(
                "Could not find project version to use as tag. Please specify a tag."
            )

    repo_name = get_repo(root)
    if not repo_name:
        sys.exit("Could not find GitHub repository name.")

    repo = Repository(token=token, repo=repo_name)
    data = {
        "tag_name": tag,
        "target_commitish": branch,
        "name": tag,
        "body": notes,
        "draft": False,
        "prerelease": False,
        "generate_release_notes": not notes,
    }
    repo.create_release(data)
    print(f"Successfully created release {tag} on {repo_name}.")


def main():
    args = parse_args()
    release_on_github(
        token=args.token, branch=args.branch, tag=args.tag, notes=args.notes
    )


if __name__ == "__main__":
    main()



import argparse
import os
import subprocess as sp
import sys


def parse_args(args=None, namespace=None):
    parser = argparse.ArgumentParser(
        description="Automatically commit staged changes with a generated message."
    )
    parser.add_argument(
        "-d",
        "--dry-run",
        dest="dry_run",
        action="store_true",
        help="Show the commit message without committing staged changes.",
    )
    return parser.parse_args(args=args, namespace=namespace)


def gcommit(dry_run: bool) -> None:
    if not os.getenv("GEMINI_API_KEY"):
        sys.exit("Error: GEMINI_API_KEY environment variable is not set.")

    diff = sp.run(
        ["git", "diff", "--staged"], capture_output=True, text=True, check=True
    ).stdout.strip()
    if not diff:
        print("No staged changes to commit.")
        return

    print("Generating commit message...")
    process = sp.run(
        [
            "gemini",
            "-p",
            "Write a concise Conventional Commit message for this diff. Output ONLY the message.",
        ],
        input=diff,
        capture_output=True,
        text=True,
        check=True,
    )
    msg = process.stdout.strip()
    if not msg:
        print("Failed to generate a message.")
        return
    print(f"Commit message: {msg}")
    if dry_run:
        print("Changes are not committed due to the dry run mode.")
        return
    sp.run(["git", "commit", "-m", msg], check=True)


def main():
    args = parse_args()
    gcommit(args.dry_run)


if __name__ == "__main__":
    main()



from unittest.mock import patch
import pytest
from aiutil.pyscript.etc_sysctl import etc_sysctl


@patch("subprocess.run")
def test_etc_sysctl_apply(mock_run, tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    etc_sysctl("vm.swappiness", "10", path=conf_file, apply=True)
    assert conf_file.read_text() == "vm.swappiness = 10\n"
    mock_run.assert_called_once_with(["sysctl", "-p", str(conf_file)], check=True)


def test_etc_sysctl_invalid_value(tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    with pytest.raises(
        ValueError, match="Invalid value '5' for key 'kernel.perf_event_paranoid'"
    ):
        etc_sysctl("kernel.perf_event_paranoid", "5", path=conf_file)


def test_etc_sysctl_new_file(tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    etc_sysctl("vm.swappiness", "10", path=conf_file)
    assert conf_file.read_text() == "vm.swappiness = 10\n"


def test_etc_sysctl_append(tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    conf_file.write_text("net.ipv4.ip_forward = 0\n")
    etc_sysctl("vm.swappiness", "10", path=conf_file)
    content = conf_file.read_text()
    assert "net.ipv4.ip_forward = 0" in content
    assert "vm.swappiness = 10" in content


def test_etc_sysctl_overwrite(tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    conf_file.write_text("vm.swappiness = 60\n")
    etc_sysctl("vm.swappiness", "10", path=conf_file)
    assert conf_file.read_text() == "vm.swappiness = 10\n"


def test_etc_sysctl_overwrite_with_spaces(tmp_path):
    conf_file = tmp_path / "sysctl.conf"
    conf_file.write_text("  vm.swappiness   = 60  \n")
    etc_sysctl("vm.swappiness", "10", path=conf_file)
    assert conf_file.read_text() == "vm.swappiness = 10\n"



"""Run commands without the need to enter password (repeatedly)."""

import argparse
import base64
import datetime
import getpass
import json
import math
from pathlib import Path
import sys
import time
from typing import Iterable
import pexpect
import yaml

CONFIG_DIR = Path.home() / ".config" / "apass"
PATH_CONFIG = CONFIG_DIR / "profile.json"
PATH_PROMPTS = CONFIG_DIR / "prompts.yml"

USER = getpass.getuser()
FORMAT = "%Y-%m-%d %H:%M:%S.%f"


def _get_prompts() -> dict[str, str]:
    if not PATH_PROMPTS.is_file():
        CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        with PATH_PROMPTS.open("w") as fout:
            yaml.dump({}, fout, default_flow_style=False)
    # read prompts
    with PATH_PROMPTS.open("r") as f:
        prompts = yaml.safe_load(f) or {}
    return {k: v.format(USER=USER) for k, v in prompts.items()}


def _get_password(timeout: float) -> str:
    if PATH_CONFIG.is_file():
        token = json.loads(PATH_CONFIG.read_text())
        time_token = datetime.datetime.strptime(token["time"], FORMAT)
        if (datetime.datetime.now() - time_token).total_seconds() <= timeout:
            return base64.b64decode(token["password"]).decode()
    token_time = datetime.datetime.now()
    passwd = getpass.getpass("Please enter your password: ")
    token = {
        "password": base64.b64encode(passwd.encode()).decode(),
        "time": token_time.strftime(FORMAT),
    }
    PATH_CONFIG.parent.mkdir(parents=True, exist_ok=True)
    PATH_CONFIG.write_text(json.dumps(token, indent=4))
    return passwd


def _apass(command: Iterable[str], passwd: str):
    command = " ".join(command)
    prompts = _get_prompts()
    if not command:
        if not prompts:
            sys.exit(
                f"Error: No command is provided and no prompts are configured in {PATH_PROMPTS}."
            )
        command = next(iter(prompts))
    if command not in prompts:
        sys.exit(f"Error: Prompt for command '{command}' not found in {PATH_PROMPTS}")

    print("Auto filling password for the command", command)
    child = pexpect.spawn(command)
    child.logfile_read = sys.stdout.buffer
    child.expect(prompts[command])
    time.sleep(0.3)
    child.sendline(passwd)
    child.interact()


def parse_args():
    parser = argparse.ArgumentParser(
        description="A tool to run commands without entering your password repeatedly."
    )
    parser.add_argument(
        "command",
        nargs="*",
        default=(),
        help="The command to execute (defaults to the first command in prompts configuration).",
    )
    return parser.parse_args()


def main():
    args = parse_args()
    _apass(args.command, _get_password(timeout=math.inf))


if __name__ == "__main__":
    main()
